#ifndef PURSUITFUNCTIONS_H
#define PURSUITFUNCTIONS_H


#include "MatrixOperations.h"
#include "mat.h"
#include <cmath>
#include "MyBLAS.h"
#define msg mexPrintf
#include "my_types.h"
#define error mexErrMsgTxt

using namespace std;

/* Choosing Routines */
// Pick New Atom
// OMP
void ChooseAtomOMP(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long k, double 
        &maxValue, unsigned long &maxIndex);
void ChooseAtomOMPv1(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long iK, double 
        &maxValue, unsigned long &maxIndex);
void ChooseAtomOMPv2(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long iK, double 
        &maxValue, unsigned long &maxIndex);
void ChooseAtomOMP2D(double *mDictionaryY, double *mDictionaryX, double *mResidule, unsigned long nAtomsY,
        unsigned long nAtomsX, unsigned long szSpaceY, unsigned long szSpaceX, double &maxValue,
        unsigned long &maxIndexY, unsigned long &maxIndexX, unsigned long &maxIndexXY);
int ChooseAtomOMPSwapping(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long k);
void ChooseAtomOMPMl(double *mDictionary, double *mResidule, double *pp, unsigned long nSignals, unsigned long szSpace,
        unsigned long nAtoms, unsigned long k, double &maxValue, unsigned long &maxIndex);
int ChooseAtomOMPSwappingMl(double *mDictionary, double *mResidule, double *pp, unsigned long nSignals, 
        unsigned long szSpace, unsigned long nAtoms, unsigned long k);

// OOMP
void ChooseAtomOOMP(double *mOrthogonalNew, double *vSignal, long szSpace, long nAtoms, long k, 
        double &maxValue, unsigned long &maxIndex);
int ChooseAtomOOMPSwapping(double *mOrthogonalNew, double *vSignal, long szSpace, long nAtoms, long k);
void ChooseAtomOOMPMl(double *mOrthogonalNew, double *mSignal, double *pp,  unsigned long nSignals, long szSpace, 
        long nAtoms, long k, double &maxValue, unsigned long &maxIndex);
int ChooseAtomOOMPSwappingMl(double *mOrthogonalNew, double *mSignal, double *pp,  unsigned long nSignals, 
        long szSpace, long nAtoms, long k);


// ProjMP2D
void ChooseAtomProjMP2D(
        double *mDictColumns, 
        double *mDictRows, 
        double *mResidule, 
        ulng nRows, // # rows in mResidule and # rows in mDictColumns
        ulng nColumns, // # columns in mResidule and # rows in mDictRows
        ulng nAtoms, // # columns in mDictColumns and # columns in mDictRows
        double &maxInnerProduct, 
        ulng &iAtom);
void ChooseAtomProjMP2DLoop(
        double *mDictColumns, 
        double *mDictRows, 
        double *mResidule, 
        ulng nRows, // # rows in mResidule and # rows in mDictColumns
        ulng nColumns, // # columns in mResidule and # rows in mDictRows
        ulng nAtoms, // # columns in mDictColumns and # columns in mDictRows
        double &maxInnerProduct, 
        ulng &iAtom);
void ChooseAtomSPMP2D(
        double *mDictColumns, 
        double *mDictRows, 
        double *mResidule, 
        ulng nRows, // # rows in mResidule and # rows in mDictColumns
        ulng nColumns, // # columns in mResidule and # rows in mDictRows
        ulng nAtomsColumn,
        ulng nAtomsRow,
        double &maxInnerProduct, 
        ulng &iAtomColumn,
        ulng &iAtomRow,
        ulng &iAtomColumnRow);


// Remove existing atom
void AtomToRemove(double *mBetaNew, double *vC, double *dd, long szSpace, long nBetas, unsigned long &p);
int AtomToRemoveSwapping(double *mBetaNew, double *vC, long szSpace, long nBetas);
void AtomToRemoveBOOMP(double *mBetaNew, double *vC, double *dd, long szSpace, long nBetas, unsigned long &p);
void AtomToRemoveMl(double *mBetaNew, double *mC, double *dd, long szSpace, unsigned long nSignals,
        long nBetas, unsigned long &p, double *pp);
int AtomToRemoveSwappingMl(double *mBetaNew, double *mC, long szSpace, unsigned long nSignals,
        long nBetas, double *pp);
void AtomToRemoveBOOMPMl(double *mBetaNew, double *mC, double *dd, long szSpace, unsigned long nSignals,
        long nBetas, unsigned long &p, double *pp);

/* Calculate Residule */
void CalcResiduleOMP(double *vResidule, double *vSignal, double *vOrthogonalK, int szSignal);
void CalcResiduleSwapping(double *vSignal, double *mOrthogonalDictionary, double *vResidule, long m, long n);
void CalcResiduleC(double *vSignal, double *mDictionary, double *vResidule, double *vC, long szSpace, long nBetas);
void CalcResiduleOMPMl(double *mResidule, double *mSignal, double *vOrthogonal, unsigned long szSignal, 
        unsigned long nSignals);
void CalcResiduleCMl(double *mSignal, double *mDictionary, double *mC, double *mResidule, 
       unsigned long nSignals, long szSpace, long nBetas);
void CalcResiduleSwappingMl(double *mSignal, double *mDictionary, double *mBeta, double *mResidule, 
       unsigned long nSignals, long szSpace, long nBetas);

/* Calculate Residule Norm */
double CalcResiduleNormC(double *vSignal, double *mDictionary, double *vC, long szSpace, long nBetas);
double CalcResiduleNorm(double *vSignal, double *mDictionary, double *mBeta, long szSpace, long nBetas);
double CalcResiduleNormCMl(double *mSignal, double *mDictionaryNew, double *mC, long szSpace, unsigned long nSignals, 
        long nBetas, double *pp);
double CalcNormMl( double *mResidule, double *pp, unsigned long nSignals, unsigned long szSpace);
double CalcResiduleNormMl(double *mSignal, double *mDictionary, double *mBeta, long szSignal, 
        long nSignals, unsigned long nBetas, double *pp );
double CalcResiduleNormOOMPMl( double *mResidule, double *mSignal, double *vOrthogonal, long szSignal, 
        long nSignals, double *pp );

/* Other */
void OrthogonalizeOOMP(double *mOrthogonalDictionary, long szSignal, long k, long nAtoms);
void Reorthognalize(double *mOrthogonalDictionary, long szSignal, long k, int nRepetitions);
void Reorthognalizev1(double *mOrthogonalDictionary,long szSignal,long k, int nRepetitions);
void OrthogonalizeOMP(double *mOrthogonalDictionary, double *vAtom, long szSignal,long k);
void OrthogonalizeOMPv1(double *mOrthogonalDictionary, double *vAtom, long szSignal,long k);
void CalcBiorthogonal(double *mBiorthogonal, double *vNewAtom, double *vOrthogonalAtom, double normAtom, long szSignal, long k);
void CalcBiorthogonal2(double *mBiorthogonal, double *vBetaRemoved, long szSignal, long nBetas);
void UpdateQkbTok( double *mOrthogonal, double *mDictionary, long iDeleted, long m, long n);
void UpdateQjTokb(double *mOrthogonal, double mDictionary, long m, long j, long kb);
void UpdateBetas(double *mBeta, long iDeletedAtom, long m, long n);

void ValidateIndex(double *ind, unsigned long nAtoms, unsigned long numInd);

void CalcIndexYAndX(unsigned long indexYX, unsigned long nColumnAtoms, unsigned long nRowAtoms, 
        unsigned long &iAtomColumn, unsigned long &iAtomRow);
double Mean( double *vec, int length );
void SaveToMat(double *mArray, long m, long n, char *fileName, char *varName);
int Min( int int1, int int2 );
void MinVector( double *v, unsigned long n, double &minElement, unsigned long &minIndex);
double Std( double *vec, int length );

/*
 * Start of main functions
 */

/* Choosing Routines */
// Pick New Atom
// OMP
void ChooseAtomOMP(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long k, double 
        &maxValue, unsigned long &maxIndex)
{        
    /* MATLAB CODE
     *  for p=k:N, %it can be done without for
     *      cc(p)=abs(Re*D(:,p));
     *  end      
     *  [max_c,q]=max(cc);
     */ 
    double cc = 0;
    maxIndex = k;
    maxValue = abs(RealInnerProduct( vResidule, &mDictionary[k*szSpace], szSpace));
    
    for ( long i = k+1 ; i < nAtoms; i++ )
    {
        cc = abs(RealInnerProduct( vResidule, &mDictionary[i*szSpace], szSpace));
        if( cc > maxValue )
        {
            maxValue = cc;
            maxIndex = i;
        }
    }
    
    if (maxValue == 0)
    {
        msg("Atom chosen with max(abs(Re*D(:,k:N)) == 0 it is likely to be a mistake!!\n");
    }
}


// OMP Use matrix multiplication for instead of a loop
void ChooseAtomOMPv1(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long iK, double 
        &maxValue, unsigned long &maxIndex)
{        
    /* MATLAB CODE
     *  for p=k:N, %it can be done without for
     *      cc(p)=abs(Re*D(:,p));
     *  end      
     *  [max_c,q]=max(cc);
     */ 
    
    unsigned long nInnerProducts = nAtoms - iK;
    double *vInnerProducts = new double[nInnerProducts];
    MultiplyMatrixVector(&mDictionary[iK*szSpace], vResidule, vInnerProducts, szSpace, nInnerProducts, 'T');
    
    
    double cc = 0;
    maxIndex = 0;
    maxValue = abs(vInnerProducts[0]);            
    
    for ( long i = 1 ; i < nInnerProducts; i++ )
    {
        cc = abs(vInnerProducts[i]);

        if( cc > maxValue )
        {
            maxValue = cc;
            maxIndex = i;
        }
    }
    
    if (maxValue == 0)
    {
        msg("Atom chosen with max(abs(Re*D(:,k:N)) == 0 it is likely to be a mistake!!\n");
    }
    delete [] vInnerProducts;
    vInnerProducts = 0;
    
    // Return the max index in mDictionary not vInnerProducts
    maxIndex = maxIndex + iK;
}

// OMP Use matrix multiplication for instead of a loop
void ChooseAtomOMPv2(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long iK, double 
        &maxValue, unsigned long &maxIndex)
{        
    /* MATLAB CODE
     *  for p=k:N, %it can be done without for
     *      cc(p)=abs(Re*D(:,p));
     *  end      
     *  [max_c,q]=max(cc);
     */ 
    
    unsigned long nInnerProducts = nAtoms - iK;
    double *vInnerProducts = new double[nInnerProducts];
    MultiplyMatrixVector(&mDictionary[iK*szSpace], vResidule, vInnerProducts, szSpace, nInnerProducts, 'T');
    
    
	unsigned long iMaxAbsValue = MaxAbs(vInnerProducts, nInnerProducts);
	maxValue = abs(vInnerProducts[iMaxAbsValue]);
	// Return the max index in mDictionary not vInnerProducts
	maxIndex = iMaxAbsValue + iK;
    
    if (maxValue == 0)
    {
        msg("Atom chosen with max(abs(Re*D(:,k:N)) == 0 it is likely to be a mistake!!\n");
    }
    delete [] vInnerProducts;
    vInnerProducts = 0;

}

void ChooseAtomOMP2D(double *mDictionaryY, double *mDictionaryX, double *mResidule, unsigned long nAtomsY,
        unsigned long nAtomsX, unsigned long szSpaceY, unsigned long szSpaceX, double &maxValue,
        unsigned long &maxIndexY, unsigned long &maxIndexX, unsigned long &maxIndexXY)
{
    /* MATLAB CODE
     *  cc = abs(Dy'*I*Dx);
     *  [c1,c2]=max(cc);
     *  [max_c,b2]=max(c1);
     */ 
    
    /* First multiply the two matrices that result in the matrix of smallest dimension to reduce the total 
     * number of flops.  This will be the smallest of nAtomsY * szSpaceX and szSpaceY * nAtomsX */
    unsigned long szA = nAtomsY * szSpaceX;
    unsigned long szB = szSpaceY *nAtomsX;
    
    /* Declare the innerproduct matrix as its size does nAtomsY*nAtomsX not depend on the order of the calculation */
    unsigned long nInnerProducts = nAtomsY*nAtomsX;
    double *mInnerProducts = new double[nInnerProducts];
    
    if (szA <= szB)
    {
        double *mYMImage = new double[szA];
        MatrixMultiply(mDictionaryY, mResidule, mYMImage, szSpaceY, nAtomsY, szSpaceX, 'T', 'N');
        MatrixMultiply(mYMImage, mDictionaryX, mInnerProducts, nAtomsY, szSpaceX, nAtomsX, 'N', 'N');
        delete [] mYMImage;
        mYMImage = 0;
    }else{
        double *mXMImage = new double[szB];
        MatrixMultiply(mResidule, mDictionaryX, mXMImage, szSpaceY, szSpaceX, nAtomsX, 'N', 'N');
        MatrixMultiply(mDictionaryY, mXMImage, mInnerProducts, szSpaceY, nAtomsY, nAtomsX, 'T', 'N');
        delete [] mXMImage;
        mXMImage = 0;
    }
    
    maxIndexXY = MaxAbs(mInnerProducts, nInnerProducts);
    maxValue = abs(mInnerProducts[maxIndexXY]);
    delete [] mInnerProducts;
    mInnerProducts = 0;
    
    /* Get the index's in the seperate dictionaries */
    CalcIndexYAndX(maxIndexXY, nAtomsY, nAtomsX, maxIndexY, maxIndexX);

}


void ChooseAtomSPMP2D(
        double *mDictColumns, 
        double *mDictRows, 
        double *mResidule, 
        ulng nRows, // # rows in mResidule and # rows in mDictColumns
        ulng nColumns, // # columns in mResidule and # rows in mDictRows
        ulng nAtomsColumn,
        ulng nAtomsRow,
        double &maxInnerProduct, 
        ulng &iAtomColumn,
        ulng &iAtomRow,
        ulng &iAtomColumnRow)
{
    /* Chooses the atom that is closest in norm to the residule and returns
     * its index and value.
     * NOTE: Does not return the absolute value of the coefficient.
     */
    
    /* MATLAB
     *  cc=(Vx'*Re*Vy);
     *  [c1,c2]=max(abs(cc));
     *  [max_c,b2]=max(c1); %selects the atom
     *  q=[c2(b2) b2];
     *  c = cc(q(1),q(2));  
     */
    
    /* MATLAB
     *  cc=diag(V1'*Re*V2);
     *  [c1,n1]=max(abs(cc));
     */

    /* MATLAB
     *  cc=Vx'*Re*Vy;
     */
    // Initialize inner product matrix
    ulng nInnerProducts = nAtomsColumn*nAtomsRow;
    double *mInnerProducts = new double[nInnerProducts];
    // mInnerProducts = V1'*Re*V2
    MatrixMultiplyABC(mDictColumns, mResidule, mDictRows, mInnerProducts,
            nRows, nAtomsColumn, nColumns, nAtomsRow, 'T', 'N', 'N');

    /* MATLAB
     *  [c1,c2]=max(abs(cc));
     *  [max_c,b2]=max(c1); %selects the atom
     *  q=[c2(b2) b2];
     *  c = cc(q(1),q(2)); 
     */
    // <<--
        iAtomColumnRow = MaxAbs(mInnerProducts, nInnerProducts);
        maxInnerProduct = mInnerProducts[iAtomColumnRow];

        delete [] mInnerProducts;
        mInnerProducts = 0;

        /* Get the index's in the seperate dictionaries */
		// Input is nColumns nRows i.e. mInnerProducts is nAtomColumns x nAtomRows
        CalcIndexYAndX(iAtomColumnRow, nAtomsColumn, nAtomsRow, 
                iAtomColumn, iAtomRow); // CHECK THIS CALL
    // -->>
}


int ChooseAtomOMPSwapping(double *mDictionary, double *vResidule, long szSpace, long nAtoms, long k)
{        
    /* MATLAB CODE
     *  for p=k:N, %it can be done without for
     *      cc(p)=abs(Re*D(:,p));
     *  end      
     *  [max_c,q]=max(cc);
     */ 
    double maxValue = 0;
    unsigned long maxIndex = 0;
    ChooseAtomOMP(mDictionary, vResidule, szSpace, nAtoms, k, maxValue, maxIndex);    
    return maxIndex;
}

// OOMP
void ChooseAtomOOMP(double *mOrthogonalNew, double *vSignal, long szSpace, long nAtoms, long k, 
        double &maxValue, unsigned long &maxIndex)
{
    /*  for p=k:N
     *      c(p)=norm(Q(:,p));
     *      if c(p)>tol1 
     *          cc(p)=abs(f*Q(:,p))/c(p);
     *      end
     *  end
     *  [max_c,q]=max(cc);
     */  
    const double tolerance = 1e-7;
    double c = tolerance,cc = 0;
    maxIndex = k;
    maxValue = 0; // Just so its clear that it exits with zero if c !> tolerance
    while ( c <= tolerance && maxIndex != nAtoms)
    {           
        c = Norm(&mOrthogonalNew[maxIndex*szSpace], szSpace);
        maxIndex++;
    }                

    maxIndex--;
    
    if( c > tolerance )
    {            
        maxValue = abs(RealInnerProduct( vSignal, &mOrthogonalNew[maxIndex*szSpace], szSpace))/c;
    }else{
        msg("Error choosing new atom! Norm of all orthogonal atoms is less than tolerance\n");
    }

    /* If we enter this loop then we have satisfied the tolerance, so no more verification required 
     * as we have a max value even if c > tolerance again
     */
    for ( long i = maxIndex+1 ; i < nAtoms; i++ )
    {
        c = Norm(&mOrthogonalNew[i*szSpace], szSpace);
        if (c > tolerance)
        {
            cc = abs(RealInnerProduct( vSignal, &mOrthogonalNew[i*szSpace], szSpace))/c;
            if( cc > maxValue )
            {
                maxValue = cc;
                maxIndex = i;
            }            
        }
    }
    
    if (maxValue == 0)
    {
        msg("Atom chosen with max(abs(f*Q(:,p))/c(p)) == 0 it is likely to be a mistake!!\n");
    }    
}

int ChooseAtomOOMPSwapping(double *mOrthogonalNew, double *vSignal, long szSpace, long nAtoms, long k)
{
    /*  for p=k:N
     *      c(p)=norm(Q(:,p));
     *      if c(p)>tol1 
     *          cc(p)=abs(f*Q(:,p))/c(p);
     *      end
     *  end
     *  [max_c,q]=max(cc);
     */  
    double maxValue = 0;
    unsigned long maxIndex = 0;    
    ChooseAtomOOMP(mOrthogonalNew, vSignal, szSpace, nAtoms, k, maxValue, maxIndex);
    return maxIndex;
}


void ChooseAtomOMPMl(double *mDictionary, double *mResidule, double *pp, unsigned long nSignals, unsigned long szSpace,
        unsigned long nAtoms, unsigned long k, double &maxValue, unsigned long &maxIndex)
{
    /* MATLAB CODE
     *  for p=k:N, %it can be done without for
     *      cp(:,p)=abs(Re*D(:,p)); 
     *      cc(p)=pp*cp(:,p);
     *  end      
     *  [max_c,q]=max(cc);
     */    
    double *temp = new double[nSignals];
    maxIndex = k;
	//SaveToMat(mResidule, nSignals, szSpace, "dict.mat", "DTest");
    MultiplyMatrixVector(mResidule, &mDictionary[k*szSpace], temp, nSignals, szSpace, 'N');
	for (unsigned long j = 0; j< nSignals; j++)
	{
		temp[j] = abs(temp[j]);
	}
    maxValue = RealInnerProduct( pp, temp, nSignals);
    
    for ( unsigned long i = k+1 ; i < nAtoms; i++ )
    {
        MultiplyMatrixVector(mResidule, &mDictionary[i*szSpace], temp, nSignals, szSpace, 'N');
		for (unsigned long j = 0; j< nSignals; j++)
		{
			temp[j] = abs(temp[j]);
		}
        double cc = RealInnerProduct( pp, temp, nSignals);        
        if( cc > maxValue )
        {
            maxValue = cc;
            maxIndex = i;
        }
    }
    
    delete[] temp;
    temp = 0;
    
    if (maxValue == 0)
    {
        msg("Atom chosen with max(abs(Re*D(:,k:N)) == 0 it is likely to be a mistake!!\n");
    }    
}

int ChooseAtomOMPSwappingMl(double *mDictionary, double *mResidule, double *pp, unsigned long nSignals, 
        unsigned long szSpace, unsigned long nAtoms, unsigned long k)
{
    /* MATLAB CODE
     *  for p=k:N, %it can be done without for
     *      cp(:,p)=abs(Re*D(:,p)); 
     *      cc(p)=pp*cp(:,p);
     *  end      
     *  [max_c,q]=max(cc);
     */  
    double maxValue = 0;
    unsigned long maxIndex = 0;
    ChooseAtomOMPMl(mDictionary, mResidule, pp, nSignals, szSpace, nAtoms, k, maxValue, maxIndex);
    return maxIndex;
}

void ChooseAtomOOMPMl(double *mOrthogonalNew, double *mSignal, double *pp,  unsigned long nSignals, long szSpace, 
        long nAtoms, long k, double &maxValue, unsigned long &maxIndex)
{
    
    /*  for p=k:N
     *      c(p)=norm(Q(:,p));
     *      if c(p)>tol1 
     *          cp(:,p)=abs(f*Q(:,p))/c(p); 
     *          cc(p)=pp*cp(:,p);
     *      end
     *  end
     *  [max_c,q]=max(cc);
     */
    const double tolerance = 1e-7;
    double c = tolerance,cc = 0;
    maxIndex = k;
    double *temp = new double[nSignals];
    maxValue = 0; // Just so its clear that it exits with zero if c !> tolerance
    
    /* c(p)=norm(Q(:,p)); */
    while ( c <= tolerance && maxIndex != nAtoms)
    {           
        c = Norm(&mOrthogonalNew[maxIndex*szSpace], szSpace);      
        maxIndex++;
    }                

    maxIndex--;

    if( c > tolerance )
    {            
        /* cp(:,p)=abs(f*Q(:,p))/c(p); */
        MultiplyMatrixVector(mSignal, &mOrthogonalNew[maxIndex*szSpace], temp, nSignals, szSpace, 'N');
        for (unsigned long j = 0; j< nSignals; j++)
        {
            temp[j] = abs(temp[j]);
        }
        /* cc(p)=pp*cp(:,p); */
        maxValue = RealInnerProduct( pp, temp, nSignals)/c;   
    }else{
        msg("Error choosing new atom! Norm of all orthogonal atoms is less than tolerance\n");
    }		

    /* If we enter this loop then we have satisfied the tolerance, so no more verification required 
     * as we have a max value even if c > tolerance again
     */
    for ( long i = maxIndex+1 ; i < nAtoms; i++ )
    {
        c = Norm(&mOrthogonalNew[i*szSpace], szSpace);
        if (c > tolerance)
        {
            /* cp(:,p)=abs(f*Q(:,p))/c(p); */
            MultiplyMatrixVector(mSignal, &mOrthogonalNew[i*szSpace], temp, nSignals, szSpace, 'N');
            for (unsigned long j = 0; j< nSignals; j++)
            {
                temp[j] = abs(temp[j]);
            }
            /* cc(p)=pp*cp(:,p); */
            cc = RealInnerProduct( pp, temp, nSignals)/c; 
            if( cc > maxValue )
            {
                maxValue = cc;
                maxIndex = i;
            }            
        }
    }     
        
    delete[] temp;
    temp = 0;
    
    if (maxValue == 0)
    {
        msg("Atom chosen with max(abs(f*Q(:,p))/c(p)) == 0 it is likely to be a mistake!!\n");
    }   
    
}

int ChooseAtomOOMPSwappingMl(double *mOrthogonalNew, double *mSignal, double *pp,  unsigned long nSignals, 
        long szSpace, long nAtoms, long k)
{
    double maxValue = 0;
    unsigned long maxIndex = 0;
    ChooseAtomOOMPMl(mOrthogonalNew, mSignal, pp,  nSignals, szSpace, nAtoms, k, maxValue, maxIndex);   
    return maxIndex;
}


void ChooseAtomProjMP2D(
        double *mDictColumns, 
        double *mDictRows, 
        double *mResidule, 
        ulng nRows, // # rows in mResidule and # rows in mDictColumns
        ulng nColumns, // # columns in mResidule and # rows in mDictRows
        ulng nAtoms, // # columns in mDictColumns and # columns in mDictRows
        double &maxInnerProduct, 
        ulng &iAtom)
{
    /* MATLAB
     *  cc=diag(V1'*Re*V2);
     *  [c1,n1]=max(abs(cc));
     */

    /* MATLAB
     *  cc=V1'*Re*V2;
     */
    // Initialize inner product matrix
    ulng nInnerProducts = nAtoms*nAtoms;
    double *mInnerProducts = new double[nInnerProducts];
    // mInnerProducts = V1'*Re*V2
    MatrixMultiplyABC(mDictColumns, mResidule, mDictRows, mInnerProducts,
            nRows, nAtoms, nColumns, nAtoms, 'T', 'N', 'N');
    /* MATLAB
     *  [c1,n1]=max(abs(diag(cc)));
     */
    iAtom = MaxAbsDiag(mInnerProducts, nAtoms); 
    ulng iInnerProduct = iAtom*nAtoms + iAtom;
    maxInnerProduct = mInnerProducts[iInnerProduct];
    
    delete [] mInnerProducts;
    mInnerProducts = 0;
}


void ChooseAtomProjMP2DLoop(
        double *mDictColumns, 
        double *mDictRows, 
        double *mResidule, 
        ulng nRows, // # rows in mResidule and # rows in mDictColumns
        ulng nColumns, // # columns in mResidule and # rows in mDictRows
        ulng nAtoms, // # columns in mDictColumns and # columns in mDictRows
        double &maxInnerProduct, 
        ulng &iAtom)
{
    /* MATLAB
     *  for n=1:N1
     *      cc(n)=V1(:,n)'*Re*V2(:,n);
     *  end
     */
    double *vInnerProducts = new double[nAtoms];
    for (ulng iInnerProduct = 0; iInnerProduct < nAtoms; iInnerProduct ++)
    {
        /* Index of first element of atom in mDictColumns and mDictRows */
        ulng iMDictColumns = iInnerProduct * nRows;
        ulng iMDictRows = iInnerProduct * nColumns;
        
        vInnerProducts[iInnerProduct] = VectorMatrixVector(
                &mDictColumns[iMDictColumns], mResidule, 
                &mDictRows[iMDictRows], nRows, nColumns);       
    }
    
    /* MATLAB
     *  [c1,n1]=max(abs(cc));
     */
    iAtom = MaxAbs(vInnerProducts, nAtoms);
    maxInnerProduct = vInnerProducts[iAtom];
    
    delete [] vInnerProducts;
    vInnerProducts = 0;    
}



// Choose atom to remove
void AtomToRemove(double *mBetaNew, double *vC, double *dd, long szSpace, long nBetas, unsigned long &p)
{
        /*  for i=1:k  
         *      dd(i)=beta(:,i)'*beta(:,i);
         *  end	
         */
        for ( long i = 0; i < nBetas; i++ )
        {
            dd[i] = RealInnerProduct( &mBetaNew[i*szSpace], &mBetaNew[i*szSpace], szSpace );
        }

        /* [Cp,p]=min((C.*conj(C))./dd); % p-th atom to be removed */
        double minTemp;
        double Cp = vC[0]*vC[0]/dd[0];
        p = 0;
        for ( long i = 1; i < nBetas; i++ )
        {
            minTemp = vC[i]*vC[i]/dd[i];
            if ( minTemp < Cp )
            {
                Cp = minTemp;
                p = i;
            }
        }         
}

int AtomToRemoveSwapping(double *mBetaNew, double *vC, long szSpace, long nBetas)
{
    double *dd = new double[nBetas];
    unsigned long p = 0;
    AtomToRemove(mBetaNew, vC, dd, szSpace, nBetas, p);
    
    /* Free the memory */
    delete[] dd;
    dd = 0; // For safety
    
    return p;
}

void AtomToRemoveBOOMP(double *mBetaNew, double *vC, double *dd, long szSpace, long nBetas, unsigned long &p)
{
    AtomToRemove(mBetaNew, vC, dd, szSpace, nBetas, p);
}

void AtomToRemoveMl(double *mBetaNew, double *mC, double *dd, long szSpace, unsigned long nSignals,
        long nBetas, unsigned long &p, double *pp)
{
        /*  for i=1:k  
         *      dd(i)=beta(:,i)'*beta(:,i);
         *  end	
         */
        for ( long i = 0; i < nBetas; i++ )
        {
            dd[i] = RealInnerProduct( &mBetaNew[i*szSpace], &mBetaNew[i*szSpace], szSpace );
        }

        /* [Cp,p]=min(wt*(C.*conj(C))./dd); % p-th atom to be removed */
		double Cp = 0;
		for ( unsigned long i = 0; i < nSignals; i++)
		{
			Cp = Cp + pp[i]*mC[i]*mC[i]/dd[0];
		}
        double minTemp = 0;
        p=0;                

		for ( unsigned long i = 1; i < nBetas; i++ )
		{
			minTemp = 0;
			for ( unsigned long j = 0; j < nSignals; j++ ) 
			{
				minTemp = minTemp + pp[j]*mC[i*nSignals+j]*mC[i*nSignals+j]/dd[i];
			}
                if ( minTemp < Cp )
                {
                    Cp = minTemp;
                    p = i;
                }
		}                 
}

int AtomToRemoveSwappingMl(double *mBetaNew, double *mC, long szSpace, unsigned long nSignals,
        long nBetas, double *pp)
{
    double *dd = new double[nBetas];
    unsigned long p = 0;
    AtomToRemoveMl(mBetaNew, mC, dd, szSpace, nSignals, nBetas, p, pp);
    
    /* Free the memory */
    delete[] dd;
    dd = 0; // For safety
    
    return p;
}

void AtomToRemoveBOOMPMl(double *mBetaNew, double *mC, double *dd, long szSpace, unsigned long nSignals,
        long nBetas, unsigned long &p, double *pp)
{
    AtomToRemoveMl(mBetaNew, mC, dd, szSpace, nSignals, nBetas, p, pp);
}


/* Other Routines */
void OrthogonalizeOOMP(double *mOrthogonalDictionary, long szSignal, long k, long nAtoms)
{    
    /* orthogonalization of Q(:,p) wrt Q(:,k)
     * MATLAB CODE     
     *      for p=k+1:N
     *          Q(:,p)=Q(:,p)-(Q(:,k)'*Q(:,p))*Q(:,k);
     *      end
     */  
    
    /* Calculate
     * Q(:,k+1:end)=Q(:,k+1:end)-Q(:,k)*(Q(:,k+1:end)'Q(:,k))';
     */

    /* The size of the orthogonal matrices we multiply */        
    long temp = nAtoms - (k + 1);
    /* Declare dynamic arrays remember to delete[] after use */
    double *alpha = new double[temp];

    /* Calculate
     * alpha = Q(:,k+1:end)'Q(:,k);
     */
    MultiplyMatrixVector(&mOrthogonalDictionary[(k+1)*szSignal],&mOrthogonalDictionary[k*szSignal], alpha, 
            szSignal, temp, 'T');    
    /* Calculate
     * Q(:,k+1:end)=Q(:,k+1:end)-Q(:,k)*alpha';
     */
    OuterProduct(&mOrthogonalDictionary[(k+1)*szSignal], &mOrthogonalDictionary[k*szSignal], alpha, szSignal, temp);
    /* Free the memory */
    delete[] alpha;
    alpha = 0; // For safety        
}

void Reorthognalize(double *mOrthogonalDictionary,long szSignal,long k, int nRepetitions)
{
     /* MATLAB CODE
      * for l=1:nRepetitions
      *     for p=1:k-1
      *              Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p);
      *      end
      * end
     */
    
    /* Declare dynamic arrays remember to delete[] after use */
    double *alpha = new double[k];

    for ( int l = 0; l < nRepetitions; l++ )
    {        
        /* Calculate
         * alpha = Q(:,1:k-1)'*Q(:,k);
         */
        MultiplyMatrixVector(mOrthogonalDictionary,&mOrthogonalDictionary[(k*szSignal)], alpha, szSignal, k, 'T');
        
        /* Calculate
         *  for p=1:k-1
         *      Q(:,k)=Q(:,k)-alpha(p)*Q(:,p);
         *  end
         */
        for ( long i = 0; i < k ; i++)
        {
            AddVectors(&mOrthogonalDictionary[(k*szSignal)], &mOrthogonalDictionary[(i*szSignal)], -alpha[i], szSignal);
        }            
    }
    
    /* Free the memory */
    delete[] alpha;
    alpha = 0; // For safety
}


void Reorthognalizev1(double *mOrthogonalDictionary,long szSignal,long k, int nRepetitions)
{
     /* MATLAB CODE
      * for l=1:nRepetitions
      *     for p=1:k-1
      *              Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p);
      *      end
      * end
     */
    
    /* Declare dynamic arrays remember to delete[] after use */
    double *alpha = new double[k];
	double *vProjection  = new double[szSignal];

    for ( int l = 0; l < nRepetitions; l++ )
    {        
        /* Calculate
         * alpha = Q(:,1:k-1)'*Q(:,k);
         */
        MultiplyMatrixVector(mOrthogonalDictionary,&mOrthogonalDictionary[(k*szSignal)], alpha, szSignal, k, 'T');
        
		/* Calculate
		 * vProjection = Q(:,1:k-1)*alpha;
		 */
		MultiplyMatrixVector(mOrthogonalDictionary, alpha, vProjection, szSignal, k, 'N');

		/* Calculate
		 *	Q(:,k) = Q(:,k) - vProjection;
		 */
		AddVectors(&mOrthogonalDictionary[k*szSignal],vProjection,-1,szSignal);         
    }
    
    /* Free the memory */
    delete[] alpha;
	delete[] vProjection;
    alpha = 0; // For safety
	vProjection = 0;
}


void OrthogonalizeOMP(double *mOrthogonalDictionary, double *vAtom, long szSignal,long k)
{
    /* MATLAB CODE
    *  for p=1:k-1
    *      Q(:,k)=D(:,k)-(Q(:,p)'*D(:,k))*Q(:,p);
    *  end
    */
    
    /* Declare dynamic arrays remember to delete[] after use */
    double *alpha = new double[k];
     
    /* Calculate
     * alpha = Q(:,1:k-1)'*D(:,k);
     */
    MultiplyMatrixVector(mOrthogonalDictionary, vAtom, alpha, szSignal, k, 'T');

    /* Calculate
     *  for p=1:k-1
     *      Q(:,k)=D(:,k)-alpha(p)*Q(:,p);
     *  end
     */
    for ( long i = 0; i < k ; i++)
    {
        for ( long j = 0; j < szSignal; j++ )
        {
            mOrthogonalDictionary[(k*szSignal)+j] = vAtom[j] - alpha[i]*mOrthogonalDictionary[(i*szSignal)+j];

        }
    }     
    
    /* Free the memory */
    delete[] alpha;
    alpha = 0; // For safety
}


void OrthogonalizeOMPv1(double *mOrthogonalDictionary, double *vAtom, long szSignal,long k)
{
    /* MATLAB CODE
    *  for p=1:k-1
    *      Q(:,k)=D(:,k)-(Q(:,p)'*D(:,k))*Q(:,p);
    *  end
    */
    
    /* Declare dynamic arrays remember to delete[] after use */
    double *alpha = new double[k];
     
    /* Calculate
     * alpha = Q(:,1:k-1)'*D(:,k);
     */
    MultiplyMatrixVector(mOrthogonalDictionary, vAtom, alpha, szSignal, k, 'T');

	/* Calculate
	 *	vProjection = Q(:,1:k-1)'*alpha;
	 */
	MultiplyMatrixVector(mOrthogonalDictionary, alpha, &mOrthogonalDictionary[k*szSignal], szSignal, k, 'N');

	/* Calculate
	 *	Q(:,k) = D(:,k) - vProjection;
	 */
	AddVectors1(&mOrthogonalDictionary[k*szSignal],vAtom,-1,szSignal);


    /* Calculate
     *  for p=1:k-1
     *      Q(:,k)=D(:,k)-alpha(p)*Q(:,p);
     *  end
     */
/*    for ( long i = 0; i < k ; i++)
    {
        for ( long j = 0; j < szSignal; j++ )
        {
            mOrthogonalDictionary[(k*szSignal)+j] = vAtom[j] - alpha[i]*mOrthogonalDictionary[(i*szSignal)+j];

        }
    }    */ 
    
    /* Free the memory */
    delete[] alpha;
    alpha = 0; // For safety
}


void CalcBiorthogonal(double *mBiorthogonal, double *vNewAtom, double *vOrthogonalAtom, double normAtom, long szSignal, long k)
{
    // k is an index beta is size szSignal x k on entry and szSignal x k+1 on exit
    /* Compute biorthogonal functions beta from 1 to k-1
     * MATLAB CODE
     * beta=beta-Q(:,k)*(D(:,k)'*beta)/nork;
     * Calculate the kth biorthogonal function
     * beta(:,k)=Q(:,k)/nork; % kth biorthogonal function
     */         
    if(k>0)
    {
        double *alpha = new double[k];

        /* alpha = (beta*D(:,k)'); */ 
        MultiplyMatrixVector(mBiorthogonal,vNewAtom, alpha, szSignal, k, 'T');
        /* alpha = alpha/nork; */
        ScaleVector(alpha,1/normAtom,k);            
        /* beta = beta-Q(:,k)*alpha'; */
        OuterProduct(mBiorthogonal, vOrthogonalAtom, alpha, szSignal, k);

        /* Free the memory */
        delete[] alpha;
        alpha = 0; // For safety    
    }

    /* beta(:,k) = Q(:,k); */
    CopyElements(vOrthogonalAtom,&mBiorthogonal[k*szSignal],szSignal);

    /* beta(:,k) = beta(:,k)/nork; */
    ScaleVector(&mBiorthogonal[k*szSignal],1/normAtom,szSignal);            
}

void CalcBiorthogonal2(double *mBiorthogonal, double *vBetaRemoved, long szSignal, long nBetas)
{
    /* Compute biorthogonal functions beta from 1 to k-1
     * MATLAB CODE
     * beta=beta-b*(b'*beta);
     */  
    double *alpha  = new double[nBetas];

    /* alpha = (beta*b'); */ 
    MultiplyMatrixVector(mBiorthogonal,vBetaRemoved, alpha, szSignal, nBetas, 'T');          
    /* beta = beta-b*alpha'; */
    OuterProduct(mBiorthogonal, vBetaRemoved, alpha, szSignal, nBetas);

    /* Free the memory */
    delete[] alpha;
    alpha = 0; // For safety       
}

void UpdateBetas(double *mBeta, long iDeletedAtom, long m, long n)
{
    /* MATLAB code
     * b=beta(:,j)/norm(beta(:,j));
     * beta(:,j)=[];
     * beta=beta-b*(b'*beta);
     */    
    double *vTemp = new double[m];
    
    /* b=beta(:,j);/norm(beta(:,j)); */
    CopyElements(&mBeta[iDeletedAtom*m], vTemp, m);
    Normalize(vTemp, m, 0);
    
    /* beta(:,j)=[]; */
    RemoveColumn( mBeta, iDeletedAtom, m, n);

    /* beta=beta-b*(b'*beta); */
    CalcBiorthogonal2(mBeta, vTemp, m, n-1);
    
    /* Free the memory */
    delete [] vTemp;
    vTemp = 0; // For safety
}

void UpdateQjTokb(double *mOrthogonal, double *mDictionary, long m, long j, long kb)
{    
    double *mRotate = new double[kb*kb];
    double DC,DS;
    /* Original Code
     * R=psin(:,1:kb)'*D(:,1:kb);
     * However we want R transposed for applying rotations
     * R= D(:,1:kb)'*psin(:,1:kb);
     */
    MatrixMultiply(mDictionary, mOrthogonal, mRotate, m, kb, kb, 'T', 'N');
 
    /* Original Code
     * dummy=R(:,j);R(:,j)=[];R(:,kb)=dummy; 
     * Modified COde
     * dummy=R(j,:);R(j,:)=[];R(kb,:)=dummy;
     */
    MoveRowToEnd( &mRotate[j+1], &mRotate[j], kb - 1 - j, kb);
    
    /* Original Code
     *  for i=j:kb-1
     *      p=[i i+1];
     *      [G,R(p,i)]=planerot(R(p,i)); %find appropriate Givens rotation
     *      R(p,i+1:kb)=G*R(p,i+1:kb);   %apply Givens rotation to R(p,i+1:k)
     *      psin(:,p)=psin(:,p)*G';      %apply Givens rotation to psin(:,p)
     *  end
     * Modified Code
     *  for i=j:kb-1
     *      p=[i i+1];     
     *      [G,R(i,p)]=planerot(R(i,p)); %find appropriate Givens rotation
     *      R(i+1:kb,p)=G*R(i+1:kb,p)';   %apply Givens rotation to R(i+1:k,p)
     *      psin(:,p)=psin(:,p)*G';      %apply Givens rotation to psin(:,p)
     *  end
     */
    for ( long i = j; i < kb - 1; i++ )
    {
        PlaneRotate( &mRotate[i*kb+i],&mRotate[(i+1)*kb+i],&DC,&DS);
        ApplyPlaneRotate( &mRotate[i*kb+i+1],&mRotate[(i+1)*kb+i+1] , kb -1 -i, DC, DS);
        ApplyPlaneRotate( &mOrthogonal[i*m],&mOrthogonal[(i+1)*m] ,m, DC, DS);
    }

    /* Free the memory */
    delete[] mRotate;
    mRotate = 0; // For safety   
}

void UpdateQkbTok( double *mOrthogonal, double *mDictionary, long iDeleted, long m, long n)
{    
    /* MATLAB Code
     *  ud=psin(:,kb);psin(:,kb)=[];
     *  for i=kb:k-1
     *      psin(:,i)=psin(:,i)+(ud'*D(:,i))*ud;
     *  end 
     *  psin(:,k)=ud*(D(:,k)'*ud);
     */    
    double scaleFactor;
    /* Declare pointers to dynamic arrays remember to delete[] after use */
    double *vTemp = new double[m];
    double *vAlpha = new double[n-1-iDeleted];
    
    /* ud=psin(:,kb); */
    CopyElements(&mOrthogonal[iDeleted*m],vTemp,m);
            
    /* psin(:,kb)=[]; */
    RemoveColumn(mOrthogonal,iDeleted,m,n);
    
    /* vAlpha = (D(:,kb:k-1)'*ud); */ 
    MultiplyMatrixVector( &mDictionary[iDeleted*m], vTemp, vAlpha, m, (n-1-iDeleted), 'T');
    
    /* MATLAB Code
     * for i=kb:k-1
     *      psin(:,i)=psin(:,i)+vAlpha(i)*ud; 
     *  end 
     */
    for( long i = iDeleted; i < n-1; i++ )
    {        
        for( long j = 0; j < m; j++ )
        {
            mOrthogonal[(i*m)+ j] = mOrthogonal[(i*m)+j] + vAlpha[i-iDeleted]*vTemp[j];
        }  
    }
    
    /* Free the memory */
    delete[] vAlpha;
    vAlpha = 0; // For safety   
    
    /* scaleFactor = (D(:,k)'*ud); */
    scaleFactor = RealInnerProduct(&mDictionary[(n-1)*m], vTemp, m);
    /* psin(:,k)=ud; */
    CopyElements(vTemp, &mOrthogonal[(n-1)*m], m);
    /* psin(:,k) = psin(:,k)*scaleFator */
    ScaleVector(&mOrthogonal[(n-1)*m], scaleFactor, m);
        
    /* Free the memory */
    delete[] vTemp;
    vTemp = 0; // For safety      
}


// Calculate the residule
void CalcResiduleOMP(double *vResidule, double *vSignal, double *vOrthogonalK, int szSignal)
{
    /* Calculate the residule
     * MATLAB CODE
     * Re=Re-f*Q(:,k)*Q(:,k)';
     */
    double alpha;    
    alpha = RealInnerProduct(vOrthogonalK,vSignal,szSignal);    
    AddVectors(vResidule,vOrthogonalK,-alpha,szSignal);   
}

void CalcResiduleSwapping(double *vSignal, double *mOrthogonalDictionary, double *vResidule, long m, long n)
{
    /*  Re=f-f*Q*Q';
     */
    double *vTemp = new double[m];
    
    /* vTemp = Q'*f; */
    MultiplyMatrixVector(mOrthogonalDictionary, vSignal, vTemp, m, n, 'T');
    
    /* Re = Q*vTemp; */
    MultiplyMatrixVector(mOrthogonalDictionary, vTemp, vResidule, m, n, 'N');   
        
    /* Re = f - Re; */
    AddVectors1(vResidule, vSignal, -1, m);

    /* Free the memory */
    delete[] vTemp;
    vTemp = 0; // For safety
}

void CalcResiduleC(double *vSignal, double *mDictionary, double *vResidule, double *vC, long szSpace, long nBetas)
{
    /*  vResidule=f'-D(:,1:k)*C'); */
    MultiplyMatrixVector(mDictionary, vC, vResidule, szSpace, nBetas, 'N');
    AddVectors(vResidule, vSignal, -1, szSpace);
}

void CalcResiduleOMPMl(double *mResidule, double *mSignal, double *vOrthogonal, unsigned long szSignal, 
        unsigned long nSignals)
{
    /* Calculate the residule
     * MATLAB CODE
     * Re=Re-f*Q(:,k)*Q(:,k)';
     */
    double *vAlpha = new double[nSignals];
    MultiplyMatrixVector(mSignal, vOrthogonal, vAlpha, nSignals, szSignal, 'N');    
    OuterProduct(mResidule, vAlpha, vOrthogonal,  nSignals, szSignal);    
    delete[] vAlpha;
    vAlpha = 0;
}

void CalcResiduleCMl(double *mSignal, double *mDictionary, double *mC, double *mResidule, 
       unsigned long nSignals, long szSpace, long nBetas)
{
    /*  Re=f-C*D(:,1:size(beta,2))'; */    
    // Re = C*D(:,1:size(beta,2))';
    MatrixMultiply( mC, mDictionary, mResidule, nSignals, nBetas, szSpace, 'N', 'T');
    // Re = f - Re
    AddVectors1(mResidule, mSignal, -1, szSpace*nSignals);
}

void CalcResiduleSwappingMl(double *mSignal, double *mDictionary, double *mBeta, double *mResidule, 
       unsigned long nSignals, long szSpace, long nBetas)
{
    /*  C=f*beta;
     *  Re=f-C*D(:,1:size(beta,2))';
     */    
    // C=f*beta;
    double *mC = new double[nSignals*nBetas];
    MatrixMultiply( mSignal, mBeta, mC, nSignals, szSpace, nBetas, 'N', 'N');
    
    /*  Re=f-C*D(:,1:size(beta,2))'; */ 
    CalcResiduleCMl(mSignal, mDictionary, mC, mResidule, nSignals, szSpace, nBetas);
    
    delete[] mC;
    mC = 0;   
}


// Calculate the residule norm
double CalcResiduleNormC(double *vSignal, double *mDictionary, double *vC, long szSpace, long nBetas)
{
    /*  nor1=norm(f'-D(:,1:k)*C')*sqrt(delta); */
    double *vResidule = new double[szSpace];
    CalcResiduleC(vSignal, mDictionary, vResidule, vC, szSpace, nBetas);
    double norm =  Norm(vResidule,szSpace);
    
    /* Free the memory */
    delete[] vResidule;
    vResidule = 0; // For safety
    return norm;    
}

double CalcResiduleNorm(double *vSignal, double *mDictionary, double *mBeta, long szSpace, long nBetas)
{
    /* norm1 = norm(f'-D(:,1:k)*(f*beta)'); */
    double norm = 0, *vC = new double[nBetas];

     /* C=f*beta; */
     MultiplyMatrixVector(mBeta, vSignal, vC, szSpace, nBetas, 'T'); 
     /*  nor1=norm(f'-D(:,1:s)*c')*sqrt(delta); */
     norm = CalcResiduleNormC(vSignal, mDictionary, vC, szSpace, nBetas);
     
    /* Free the memory */
     delete[] vC;
     vC = 0; // For safety
     
     return norm;     
}

double CalcResiduleNormCMl(double *mSignal, double *mDictionary, double *mC, long szSpace, unsigned long nSignals, 
        long nBetas, double *pp)
{
    /*  Re=f-(D(:,1:size(beta,2))*C')';
     *  for i=1:mul;
     *      normre(i)=sqrt(Re(i,:)*Re(i,:)');
     *  end
     *  norf=wt*normre'*(sqrt(delta));
     */    

    double *mResidule = new double[szSpace*nSignals];
    /*  Re=f-(D(:,1:size(beta,2))*C')'; */
    CalcResiduleCMl(mSignal, mDictionary, mC, mResidule, nSignals, szSpace, nBetas);
    
    /*  for i=1:mul;
     *      normre(i)=sqrt(Re(i,:)*Re(i,:)');
     *  end
     *  norf=wt*normre'*(sqrt(delta));
     */    
    double norm = CalcNormMl( mResidule, pp, nSignals, szSpace);
    
    /* Free the memory */
    delete[] mResidule;
    mResidule = 0; // For safety
    
    return norm; 
}

double CalcNormMl( double *mResidule, double *pp, unsigned long nSignals, unsigned long szSpace)
{
    /*  for i=1:mul;
     *      normre(i)=sqrt(Re(i,:)*Re(i,:)');
     *  end
     *  norf=wt*normre'*(sqrt(delta));
     */
    double norm = 0;
    for (unsigned long i = 0; i< nSignals; i++)
    {
        norm = norm + pp[i]*Norm(&mResidule[i],szSpace,nSignals);
    }    
    return norm;
}


double CalcResiduleNormMl(double *mSignal, double *mDictionary, double *mBeta, long szSignal, 
        long nSignals, unsigned long nBetas, double *pp )
{
    /* 
     *  MATLAB CODE
     *  %stopping criterion (distance)
     *  Re1=(f*beta1*D(:,Di1(1:sdi))'-f);
     */   
    double *mResidule = new double[szSignal*nSignals];     
    CalcResiduleSwappingMl(mSignal, mDictionary, mBeta, mResidule, nSignals, szSignal, nBetas);
       
    /*  MATLAB CODE
     *  for i=1:mul
     *      normre(i)=norm(Re(i,:));
     *  end
     *  normf=pp*normre'*sqrt(delta);
     */  
     double norm = CalcNormMl( mResidule, pp, nSignals, szSignal);
    
    /* Free the memory */
    delete[] mResidule;
    mResidule = 0; // For safety
     
     return norm;   
}

double CalcResiduleNormOOMPMl( double *mResidule, double *mSignal, double *vOrthogonal, long szSignal, 
        long nSignals, double *pp )
{
    /* 
     *  MATLAB CODE
     *  %stopping criterion (distance)
     *  Re=Re-f*Q(:,k)*Q(:,k)';
     */       
    CalcResiduleOMPMl(mResidule, mSignal, vOrthogonal, szSignal, nSignals);

    /*  MATLAB CODE
     *  for i=1:mul
     *      normre(i)=norm(Re(i,:));
     *  end
     *  normf=pp*normre'*sqrt(delta);
     */  
    double norm = CalcNormMl( mResidule, pp, nSignals, szSignal);
     return norm;  
}


void ValidateIndex(double *ind, unsigned long nAtoms, unsigned long numInd)
{
    /* Check that the indexes in ind are positive and do not nAtoms.
     * Achiveved by sorting the array ind and checking that
     * the min element is greater than 0 and that the max element is less 
     * than or equal nAtoms.
     * The array ind and is avaliable in the MATLAB workspace. 
     * Copy before sorting to avoid altering MATLAB workspace variables.
     * Throw error if validation unsuccessful.
     */
    
    // Copy and sort ind array
    double *indToSort = new double[numInd];
    CopyElements(ind, indToSort, numInd);
    sort(indToSort, indToSort + numInd);
    // Validate smallest element > 0 and largest element <= nAtoms
    unsigned long smallestElement = static_cast<unsigned long>(indToSort[0]);
    unsigned long largestElement = static_cast<unsigned long>(indToSort[numInd-1]);
    if( (smallestElement <= 0 ) || (largestElement > nAtoms))
    {
            error("Demanded index's are out of dictionary!!\n");
    } 
    
    // Free dynamicaly assigned array
    delete [] indToSort;
    indToSort = 0;
          
}




/* Other helper functions */
void CalcIndexYAndX(unsigned long indexYX, unsigned long nColumnAtoms, unsigned long nRowAtoms, 
        unsigned long &iColumnAtom, unsigned long &iRowAtom)
{
    /* Calculates index's need to add 1 to return to MATLAB */
    iRowAtom = static_cast<unsigned long>(floor(static_cast<double>(indexYX)/static_cast<double>(nColumnAtoms)));
    iColumnAtom = (indexYX - (iRowAtom*nColumnAtoms));
}


int Min( int int1, int int2 )
{
    if (int1 > int2)
    {
        return int2;
    }else{
        return int1;
    }
}

void MinVector( double *v, unsigned long n, double &minElement, unsigned long &minIndex)
{
    minIndex = 0;
    minElement = 0;
    if (n > 0)
    {        
        minElement = v[minIndex];
        for ( unsigned long i = 1; i < n; i++)
        {
            if ( minElement > v[i])
            {
                minElement = v[i];
                minIndex = i;
            }
        }
    }
}                    

double Std( double *vec, int length )
{
    double mean = Mean(vec,length);
    double std = 0;
    
    for( int i = 1; i < length; i++)
    {
        std+=pow((vec[i] - mean),2);
    }
    
    return sqrt(std/(length-1));
    
}

double Mean( double *vec, int length )
{
 
    double mean = 0;
    
    for( int i = 0; i < length; i++ )
    {
        mean += vec[i];
    }
    
    return mean/length;
   
} 

void SaveToMat(double *mArray, long m, long n, char *fileName, char *varName)
{
     MATFile *pmat = matOpen(fileName, "w");
     mxArray *varToSave;
     varToSave = mxCreateDoubleMatrix(m,n, mxREAL);
     CopyElements(mArray,mxGetPr(varToSave),m*n);
     matPutVariable(pmat, varName, varToSave);
     mxDestroyArray(varToSave);
     matClose(pmat); 
}




#endif
