function  [cImApprox,nCoeff,endTime] = BlockApproxSep(cIm,method,mDictC,...
    mDictR,maxIntensity,PSNR)

% Approximates each block to a tolerance of at least tol in cIm using the 
% seperable dictionary and the method.  This will result in an image with a
% greater global PSNR than PSNRIn.
 
% Create a function handle from the method
    fhandle = str2func(method);

% Cell image dimensions
    [nRowBlocks,nColumnBlocks,nPlains] = size(cIm);
    
% Initialize return cells
    cImApprox = cell(nRowBlocks,nColumnBlocks);  
       
% Calculate the block tolerance
%     nPixels = numel(cIm{1});
    SSE = PSNRToSSE( PSNR, numel(cIm{1}), maxIntensity);
%     SSE = PSNRToBlockErrorNorm( PSNR, numel(cIm{1}), maxIntensity);   
    
% Loop through all blocks and approximate
nCoeff = 0;
startTime = tic;
for iRow = 1:nRowBlocks
    
    for iColumn = 1:nColumnBlocks
        
        % Approximate the block
            [cImApprox{iRow,iColumn},mCoeff] = fhandle(cIm{iRow,iColumn},mDictC,mDictR,SSE);
     
        % Update the number of coefficients
            nCoeff = nCoeff + numel(mCoeff);

    end
    
end  
endTime = toc(startTime);