function [f, p]=ProtSp(m,type,level,delta);

%ProtSp generate a prototype from some Cardinal B-Splines or the corresponding derivatives
%The cases are restricted to B-Splines order m=2 (linear) m=3 (quadratic) m=4(cubic). 
%For m=4 the second derivative is also an option 
%
% Usage: [f, p]=ProtSp(m,type,level,delta);
%
% Inputs:
%  m      Order of the Cardinal B-Spline. It can be 1, 2 ,3 or 4. 
%  type   1 for B-Splines, 2 for derivatives, 3 only with (m=4) for second derivative
%  level  Scale for the support (level*m) (level=1, gives the usual support=m) 
%  delta  distance between 2 consecutive evaluation points.
%
%  Outputs:
%
%   f   required prototype 
%   p   sampling points corresponding to the values f

%
Supp=m*level;
p=0:delta:Supp;
x=p/level;

switch type

case 1 
for i=1:numel(p); f(i)=ProtB(x(i),m); end
case 2
for i=1:numel(p); f(i)=ProtDB(x(i),m); end
case 3
for i=1:numel(p); f(i)=ProtDDB(x(i)); end
end
