function [psnr maxIntensity] = CalcPSNR( mImage1, mImage2, maxIntensity )
% CalcPSNR Calculates the Peak Signal to Noise Ratio (PSNR) between <latex>2<latex> matrices containing
% pixel intensity values.

%
% Usage:            psnr = CalcPSNR( mImage1, mImage2 );
%
% Inputs:
%   mImage1         matrix of pixel intensity values representing the original image
%   mImage2         matrix of pixel intensity values representing the approximated image
%   maxIntensity    maximum allowed pixel intensity, defualt is 256 (8 bit image) 
%
% Outputs:
%    psnr           the PSNR resulting from the  approximation
%

error(nargchk(2,3,nargin));

if nargin < 3
    
    % mImage1 is the original image
    bitDepth = ApproxBitdepth(mImage1);
    maxIntensity = 2^bitDepth-1;
    
end

e = double(mImage1) - double(mImage2);
[ y, x ] = size(e);
mse = sum( e(:).^2 )/( x*y );
psnr = 10*log10(((maxIntensity)^2)/mse);