function [mImage bitDepth colourType map] = LoadMImage(imagePath)

% For the purposes of approximation should always use calculated bit depth
% in case the image has been saved to a higher precision than required.
% Change this routine

% Loads an image from its path.
%   Converts indexed images to RGB or grey.
map = [];

%% Check if file exists
if exist(imagePath,'file') ~= 2
       
    error('%s not found in current directory or on MATLAB search path!',imagePath);
    
end
%% Assume its an image if imfinfo doesn't throw an exception
try
    
    sInfo = imfinfo(imagePath);
    
catch ME
    
    error('%s does not appear to be an image',imagePath);
    
end

% sImage.details = 'Image Structure';

if ~isfield(sInfo,'ColorType')
    error('%s: Illegal ColorType!',mfilename);
end

%% Check what type of image and load into structure, only grey, indexed and colorscale allowed
colourType = sInfo.ColorType;
if strcmp(colourType,'grayscale')
    mImage = imread(imagePath);
elseif strcmp(colourType,'truecolor')
    mImage = imread(imagePath);   
elseif strcmp(colourType,'indexed')    
    [mImage map] = imread(imagePath);
    
    % Check for grey of rgb colour map
    if (sum(map(:,1) - map(:,2)) > 0)...
        || (sum(map(:,1) - map(:,3)) > 0)

        % Convert to RGB
        % <<--
            mImage = ind2rgb(mImage,map);
        % -->>
    else
        % Convert to grey
        % <<--
            mImage = ind2gray(mImage,map);
        % -->>                                       
    end
        
else
    error('%s has unknown colour type!',imagePath);
end   

% Always calculate the bit depth image could be saved using a higher
% precision than necessary.
    bitDepth = ApproxBitdepth(mImage);
% - - > > #