function [mImage bitDepth colourType map] = LoadMImageW(imagePath)

% Wrapper to include loading of fits images and images stored in .mat files
% as matrices named mImage.

[pathstr, name, ext] = fileparts(imagePath);

if strcmp(ext,'.mat')
    
    matVars = whos('-file',imagePath);
    if ismember('mImage',{matVars.name})
        load(imagePath,'mImage');
    else
        error('Supplied mat file does not contain an image!');
    end
    
    if ismember('bitDepth',{matVars.name})
        load(imagePath,'bitDepth');
    else
        bitDepth = ApproxBitdepth(mImage);
    end
    
    if ismember('colourType',{matVars.name})
        load(imagePath,'colourType');
    else
        % Use function ColourType
        colourType = [];
    end
    
    if ismember('map',{matVars.name})
        load(imagePath,'map');
    else
        map = [];
    end 
    
else
    
    sInfo = imfinfo(imagePath);
    
    if strcmp(sInfo.Format,'fits')

            mImage = fitsread(imagePath,'Image');
            bitDepth = ApproxBitdepth(mImage);
            colourType = sInfo.ColorType;
            map = [];

            % Force to have positve values
    %         mImage = mImage - min(mImage(:));

    else

        [mImage bitDepth colourType map] = LoadMImage(imagePath);

    end
    
end