function Example
% Example of approximating atronomical images using OMP2D and SPMP2D.
% Use mex flag to switch between implementations.
% The execution times resulting from running this exusing a laptop with 7.7GB of 
% RAM and an intel Core 2 Duo P8600 CPU are shown below.
%   OMP2D (MATLAB implementation method = 1, useMex = 0): 89.81 seconds
%   OMP2D (MEX implementation method = 1, useMex = 1): 14.78 seconds
%   SPMP2D (MATLAB implementation method = 2, useMex = 0): 104.74 seconds
%   SPMP2D (MEX implementation method = 2, useMex = 1): 17.59 seconds

% Path to the image to be approximated
    imagePath = '../../../Corpus/Images/Hubble_Top_100/Web/eso0931a_sc.jpg';
        
% Desired method for approximation: 1 is OMP2D, 2 is SPMP2D.
    method = 1;

% Size of block partitions
    szBlock = 8;

% Use MEX file if compiled: 0 No, 1 Yes (to compile run UseMex.m)
    useMEX = 0;

% PSNR which each block is approximated to, global PSNR will be greater
% than or equal to this value
    blockPSNR = 48.01;
    
% Add required paths
    ExamplePaths(0);

% Approximate the image
    [mImApprox,PSNR,SSIM,SR,execTime,mIm] = ImApproxGreyW(imagePath,szBlock,method,useMEX,blockPSNR);

% Remove paths
    ExamplePaths(1);

% Print the results
    fprintf('\n\nResults of approximation taking %.2f seconds\n',execTime);
    % SR
    fprintf(' Sparsity Ratio (SR) -> %.2f\n',SR);
    % PSNR
    fprintf(' Peak Signal to Noise Ratio (PSNR) -> %.2fdB\n',PSNR);
    % Mean SSIM
    fprintf(' Mean Structural Similarity Index (MSSIM) -> %.2f\n',SSIM);
    % Execution Time
    
% Display figures
    figure;imshow(uint8(mIm'));
    figure;imshow(uint8(mImApprox'));